// Copyright (c) 2014 Tel-Aviv University (Israel).
// All rights reserved.
//
// This file is part of ACG (www.acg.cs.tau.ac.il).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Author(s): Michal Kleinbort <balasmic@post.tau.ac.il>
//            Oren Salzman <orenzalz@post.tau.ac.il >          

#ifndef RANDOM_MATRIX_GEN_H__RTG
#define RANDOM_MATRIX_GEN_H__RTG

#include <Eigen/Dense>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/shared_ptr.hpp>

namespace RTG
{
    /// A class used for generating a random d x d rotation matrix.
    class RandomMatrixGen 
    {
    public:
        typedef Eigen::MatrixXd                 MatrixXd;
        typedef Eigen::VectorXd                 VectorXd;
    
    private:
        typedef boost::random::mt19937          MTEng;
        typedef boost::random::normal_distribution<double> NormDist;
  
    public:
        //CTR's  
	    RandomMatrixGen(): m_dist(0.0, 1.0) {};

        /// \brief returns a shared pointer of a  d x d random rotation matrix
        boost::shared_ptr<MatrixXd> get_random_rotation_matrix(int d) const
        {
            MatrixXd A(d,d);

            const VectorXd ones (VectorXd::Ones(d));
            for (int i=0; i<d ; ++i)
                for (int j=0; j<d ; ++j)
                    A(i, j) = m_dist(m_eng);

            const Eigen::HouseholderQR<MatrixXd> qr(A) ;
            const MatrixXd Q = qr.householderQ();
      
            boost::shared_ptr<MatrixXd> p_M(new MatrixXd(d,d));
            *p_M = Q *(qr.matrixQR().diagonal().array() < 0).select(-ones, ones).asDiagonal();
      
            if(p_M->determinant() < 0)
                for (int i=0; i < d; ++i)
                (*p_M)(i,0) = -(*p_M)(i,0);

            return p_M;
        }
  

     private:
        mutable MTEng m_eng;
	    mutable NormDist m_dist;
  

    }; //RandomMatrixGen

} //namespace RTG


#endif //RANDOM_MATRIX_GEN_H__RTG